==> hlog-0.21-rc3.tar.gz: hlog-0.21-rc3/common/sig.c <==
/*
 * Copyright (c) 2022 Josef 'Jeff' Sipek <jeffpc@josefsipek.net>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

#include <jeffpc/io.h>
#include <jeffpc/nvl.h>

#include <hlog/sig.h>

#include "csv.h"

#define MAX_CSV_COLS	20

struct pota_ref {
	struct rb_node node;
	struct sig_pota_ref data;
};

struct sota_ref {
	struct rb_node node;
	struct sig_sota_ref data;
};

/*
 * Note: We do not do any sort of locking since after load, the data is
 * completely read only.
 */
static struct rb_tree pota_refs;
static struct rb_tree sota_refs;

static const char *pota_cols[] = {
	"reference", "name", "active", "entityId", "locationDesc",
};
static const char *sota_cols[] = {
	"SummitCode", "AssociationName", "RegionName", "SummitName", "AltM",
	"AltFt", "GridRef1", "GridRef2", "Longitude", "Latitude", "Points",
	"BonusPoints", "ValidFrom", "ValidTo", "ActivationCount",
	"ActivationDate", "ActivationCall",
};

static int pota_cmp(const void *va, const void *vb)
{
	const struct pota_ref *a = va;
	const struct pota_ref *b = vb;

	return str_cmp(a->data.ref, b->data.ref);
}

static int sota_cmp(const void *va, const void *vb)
{
	const struct sota_ref *a = va;
	const struct sota_ref *b = vb;

	return str_cmp(a->data.ref, b->data.ref);
}

static int __load_pota(const char **ptrs, size_t *sizes, bool *unquotes,
		       size_t nfields)
{
	struct pota_ref *entry;
	struct str *ref;
	struct str *name;
	struct str *locs;
	char dxccstr[10];
	uint16_t dxcc;
	int ret;

	/* ref */
	ref = str_dup_len(ptrs[0], sizes[0]);
	if (IS_ERR(ref))
		return PTR_ERR(ref);

	/* name */
	name = str_dup_len(ptrs[1], sizes[1]);
	if (IS_ERR(name)) {
		ret = PTR_ERR(name);
		goto err;
	}

	/* the 'active' field should be either 0 or 1 */
	if ((sizes[2] != 1) ||
	    ((ptrs[2][0] != '0') && ptrs[2][0] != '1')) {
		ret = -EINVAL;
		goto err_name;
	}

	/* dxcc is an integer */
	if (sizes[3] >= sizeof(dxccstr)) {
		ret = -EINVAL;
		goto err_name;
	}

	if (sizes[3] == 0) {
		/* use a zero when DXCC is missing */
		dxcc = 0;
	} else {
		memcpy(dxccstr, ptrs[3], sizes[3]);
		dxccstr[sizes[3]] = '\0';

		ret = str2u16(dxccstr, &dxcc);
		if (ret)
			goto err_name;
	}

	/* locs */
	locs = str_dup_len(ptrs[4], sizes[4]);
	if (IS_ERR(locs)) {
		ret = PTR_ERR(locs);
		goto err_name;
	}

	entry = malloc(sizeof(struct pota_ref));
	if (!entry) {
		ret = -ENOMEM;
		goto err_locs;
	}

	entry->data.ref = ref;
	entry->data.name = name;
	entry->data.active = ptrs[2][0] == '1';
	entry->data.dxcc = dxcc;
	entry->data.locs = locs;

	/* add to tree */
	if (rb_insert(&pota_refs, entry)) {
		ret = -EEXIST;
		goto err_locs;
	}

	return 0;

err_locs:
	str_putref(locs);
err_name:
	str_putref(name);
err:
	str_putref(ref);

	return ret;
}

static int __load_sota(const char **ptrs, size_t *sizes, bool *unquotes,
		       size_t nfields)
{
	struct sota_ref *entry;
	struct str *ref;
	struct str *name;
	char pointsstr[10];
	uint8_t points;
	int ret;

	/* ref */
	ref = str_dup_len(ptrs[0], sizes[0]);
	if (IS_ERR(ref))
		return PTR_ERR(ref);

	/* name */
	name = str_dup_len(ptrs[3], sizes[3]);
	if (IS_ERR(name)) {
		ret = PTR_ERR(name);
		goto err;
	}

	/* points is an integer */
	if (sizes[10] >= sizeof(pointsstr)) {
		ret = -EINVAL;
		goto err_name;
	}

	if (sizes[10] == 0) {
		/* use a zero when points are missing */
		points = 0;
	} else {
		memcpy(pointsstr, ptrs[10], sizes[10]);
		pointsstr[sizes[10]] = '\0';

		ret = str2u8(pointsstr, &points);
		if (ret)
			goto err_name;

		if (points > 10) {
			ret = -EINVAL;
			goto err_name;
		}
	}

	entry = malloc(sizeof(struct sota_ref));
	if (!entry) {
		ret = -ENOMEM;
		goto err;
	}

	entry->data.ref = ref;
	entry->data.name = name;
	entry->data.points = points;

	/* add to tree */
	if (rb_insert(&sota_refs, entry)) {
		ret = -EEXIST;
		goto err_name;
	}

	return 0;

err_name:
	str_putref(name);
err:
	str_putref(ref);

	return ret;
}

static int sig_load(const char *raw,
		    const char **cols, size_t ncols,
		    int (*f)(const char **, size_t *, bool *, size_t))
{
	const char *ptrs[MAX_CSV_COLS];
	size_t sizes[MAX_CSV_COLS];
	bool unquotes[MAX_CSV_COLS];
	size_t nfields;
	ssize_t ret;
	size_t len;
	size_t i;

	ret = csv_parse_line(raw, strlen(raw), &nfields,
			     ptrs, sizes, unquotes, ARRAY_LEN(ptrs));
	if (ret < 0)
		return ret;
	if (nfields != ncols)
		return -EINVAL;

	for (i = 0; i < ncols; i++)
		if (strncmp(ptrs[i], cols[i], sizes[i]))
			return -EINVAL;

	for (raw = raw + ret, len = strlen(raw); len > 0;) {
		ret = csv_parse_line(raw, len, &nfields,
				     ptrs, sizes, unquotes, ARRAY_LEN(ptrs));
		if (ret < 0)
			return ret;
		if (nfields != ncols)
			return -EINVAL;

		/* advance to the next line */
		raw += ret;
		len -= ret;

		ret = f(ptrs, sizes, unquotes, nfields);
		if (ret)
			return ret;
	}

	return 0;
}

int sig_load_pota(int dirfd, const char *fname)
{
	char *raw;
	int ret;

	rb_create(&pota_refs, pota_cmp, sizeof(struct pota_ref),
		  offsetof(struct pota_ref, node));

	raw = read_file_at(dirfd, fname);
	if (IS_ERR(raw))
		return PTR_ERR(raw);

	ret = sig_load(raw, pota_cols, ARRAY_LEN(pota_cols), __load_pota);

	free(raw);

	return ret;
}

int sig_load_sota(int dirfd, const char *fname)
{
	ssize_t ret;
	char *raw;
	char *tmp;

	rb_create(&sota_refs, sota_cmp, sizeof(struct sota_ref),
		  offsetof(struct sota_ref, node));

	raw = read_file_at(dirfd, fname);
	if (IS_ERR(raw))
		return PTR_ERR(raw);

	/* skip the first line */
	tmp = strchr(raw, '\n') + 1;

	ret = sig_load(tmp, sota_cols, ARRAY_LEN(sota_cols), __load_sota);

	free(raw);

	return ret;
}

int sig_lookup_pota(const char *ref, struct sig_pota_ref *data)
{
	struct str keyref = STR_STATIC_INITIALIZER(ref);
	struct pota_ref key = {
		.data.ref = &keyref,
	};
	struct pota_ref *found;

	found = rb_find(&pota_refs, &key, NULL);
	if (!found)
		return -ENOENT;

	data->ref = str_getref(found->data.ref);
	data->name = str_getref(found->data.name);
	data->locs = str_getref(found->data.locs);
	data->active = found->data.active;
	data->dxcc = found->data.dxcc;

	return 0;
}

int sig_lookup_sota(const char *ref, struct sig_sota_ref *data)
{
	struct str keyref = STR_STATIC_INITIALIZER(ref);
	struct sota_ref key = {
		.data.ref = &keyref,
	};
	struct sota_ref *found;

	found = rb_find(&sota_refs, &key, NULL);
	if (!found)
		return -ENOENT;

	data->ref = str_getref(found->data.ref);
	data->name = str_getref(found->data.name);
	data->points = found->data.points;

	return 0;
}

==> hlog-0.22.tar.gz: hlog-0.22/common/sig.c <==
/*
 * Copyright (c) 2022 Josef 'Jeff' Sipek <jeffpc@josefsipek.net>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

#include <jeffpc/io.h>
#include <jeffpc/nvl.h>

#include <hlog/sig.h>

#include "csv.h"

#define MAX_CSV_COLS	20

struct pota_ref {
	struct rb_node node;
	struct sig_pota_ref data;
};

struct sota_ref {
	struct rb_node node;
	struct sig_sota_ref data;
};

/*
 * Note: We do not do any sort of locking since after load, the data is
 * completely read only.
 */
static struct rb_tree pota_refs;
static struct rb_tree sota_refs;

static const char *pota_cols[] = {
	"reference", "name", "active", "entityId", "locationDesc", "latitude",
	"longitude", "grid",
};
static const char *sota_cols[] = {
	"SummitCode", "AssociationName", "RegionName", "SummitName", "AltM",
	"AltFt", "GridRef1", "GridRef2", "Longitude", "Latitude", "Points",
	"BonusPoints", "ValidFrom", "ValidTo", "ActivationCount",
	"ActivationDate", "ActivationCall",
};

static int pota_cmp(const void *va, const void *vb)
{
	const struct pota_ref *a = va;
	const struct pota_ref *b = vb;

	return str_cmp(a->data.ref, b->data.ref);
}

static int sota_cmp(const void *va, const void *vb)
{
	const struct sota_ref *a = va;
	const struct sota_ref *b = vb;

	return str_cmp(a->data.ref, b->data.ref);
}

static int __load_pota(const char **ptrs, size_t *sizes, bool *unquotes,
		       size_t nfields)
{
	struct pota_ref *entry;
	struct str *ref;
	struct str *name;
	struct str *locs;
	bool have_latlon;
	double lat, lon;
	char dxccstr[10];
	char latlon[64];
	uint16_t dxcc;
	int ret;

	have_latlon = true; /* assume we have a location */

	/* ref */
	ref = str_dup_len(ptrs[0], sizes[0]);
	if (IS_ERR(ref))
		return PTR_ERR(ref);

	/* name */
	name = str_dup_len(ptrs[1], sizes[1]);
	if (IS_ERR(name)) {
		ret = PTR_ERR(name);
		goto err;
	}

	/* the 'active' field should be either 0 or 1 */
	if ((sizes[2] != 1) ||
	    ((ptrs[2][0] != '0') && ptrs[2][0] != '1')) {
		ret = -EINVAL;
		goto err_name;
	}

	/* dxcc is an integer */
	if (sizes[3] >= sizeof(dxccstr)) {
		ret = -EINVAL;
		goto err_name;
	}

	if (sizes[3] == 0) {
		/* use a zero when DXCC is missing */
		dxcc = 0;
	} else {
		memcpy(dxccstr, ptrs[3], sizes[3]);
		dxccstr[sizes[3]] = '\0';

		ret = str2u16(dxccstr, &dxcc);
		if (ret)
			goto err_name;
	}

	/* locs */
	locs = str_dup_len(ptrs[4], sizes[4]);
	if (IS_ERR(locs)) {
		ret = PTR_ERR(locs);
		goto err_name;
	}

	/* lat */
	if ((sizes[5] == 0) || (sizes[5] >= sizeof(latlon))) {
		have_latlon = false;
	} else {
		memcpy(latlon, ptrs[5], sizes[5]);
		latlon[sizes[5]] = '\0';
		ret = str2double(latlon, &lat);
		if (ret)
			goto err_locs;
	}

	/* lon */
	if ((sizes[6] == 0) || (sizes[6] >= sizeof(latlon))) {
		have_latlon = false;
	} else {
		memcpy(latlon, ptrs[6], sizes[6]);
		latlon[sizes[6]] = '\0';
		ret = str2double(latlon, &lon);
		if (ret)
			goto err_locs;
	}

	/* grid - ignore it */

	entry = malloc(sizeof(struct pota_ref));
	if (!entry) {
		ret = -ENOMEM;
		goto err_locs;
	}

	entry->data.ref = ref;
	entry->data.name = name;
	entry->data.active = ptrs[2][0] == '1';
	entry->data.dxcc = dxcc;
	entry->data.locs = locs;
	entry->data.have_latlon = have_latlon;
	entry->data.lat = lat;
	entry->data.lon = lon;

	/* add to tree */
	if (rb_insert(&pota_refs, entry)) {
		ret = -EEXIST;
		goto err_locs;
	}

	return 0;

err_locs:
	str_putref(locs);
err_name:
	str_putref(name);
err:
	str_putref(ref);

	return ret;
}

static int __load_sota(const char **ptrs, size_t *sizes, bool *unquotes,
		       size_t nfields)
{
	struct sota_ref *entry;
	struct str *ref;
	struct str *name;
	char pointsstr[10];
	uint8_t points;
	int ret;

	/* ref */
	ref = str_dup_len(ptrs[0], sizes[0]);
	if (IS_ERR(ref))
		return PTR_ERR(ref);

	/* name */
	name = str_dup_len(ptrs[3], sizes[3]);
	if (IS_ERR(name)) {
		ret = PTR_ERR(name);
		goto err;
	}

	/* points is an integer */
	if (sizes[10] >= sizeof(pointsstr)) {
		ret = -EINVAL;
		goto err_name;
	}

	if (sizes[10] == 0) {
		/* use a zero when points are missing */
		points = 0;
	} else {
		memcpy(pointsstr, ptrs[10], sizes[10]);
		pointsstr[sizes[10]] = '\0';

		ret = str2u8(pointsstr, &points);
		if (ret)
			goto err_name;

		if (points > 10) {
			ret = -EINVAL;
			goto err_name;
		}
	}

	entry = malloc(sizeof(struct sota_ref));
	if (!entry) {
		ret = -ENOMEM;
		goto err;
	}

	entry->data.ref = ref;
	entry->data.name = name;
	entry->data.points = points;

	/* add to tree */
	if (rb_insert(&sota_refs, entry)) {
		ret = -EEXIST;
		goto err_name;
	}

	return 0;

err_name:
	str_putref(name);
err:
	str_putref(ref);

	return ret;
}

static int sig_load(const char *raw,
		    const char **cols, size_t ncols,
		    int (*f)(const char **, size_t *, bool *, size_t))
{
	const char *ptrs[MAX_CSV_COLS];
	size_t sizes[MAX_CSV_COLS];
	bool unquotes[MAX_CSV_COLS];
	size_t nfields;
	ssize_t ret;
	size_t len;
	size_t i;

	ret = csv_parse_line(raw, strlen(raw), &nfields,
			     ptrs, sizes, unquotes, ARRAY_LEN(ptrs));
	if (ret < 0)
		return ret;
	if (nfields != ncols)
		return -EINVAL;

	for (i = 0; i < ncols; i++)
		if (strncmp(ptrs[i], cols[i], sizes[i]))
			return -EINVAL;

	for (raw = raw + ret, len = strlen(raw); len > 0;) {
		ret = csv_parse_line(raw, len, &nfields,
				     ptrs, sizes, unquotes, ARRAY_LEN(ptrs));
		if (ret < 0)
			return ret;
		if (nfields != ncols)
			return -EINVAL;

		/* advance to the next line */
		raw += ret;
		len -= ret;

		ret = f(ptrs, sizes, unquotes, nfields);
		if (ret)
			return ret;
	}

	return 0;
}

int sig_load_pota(int dirfd, const char *fname)
{
	char *raw;
	int ret;

	rb_create(&pota_refs, pota_cmp, sizeof(struct pota_ref),
		  offsetof(struct pota_ref, node));

	raw = read_file_at(dirfd, fname);
	if (IS_ERR(raw))
		return PTR_ERR(raw);

	ret = sig_load(raw, pota_cols, ARRAY_LEN(pota_cols), __load_pota);

	free(raw);

	return ret;
}

int sig_load_sota(int dirfd, const char *fname)
{
	ssize_t ret;
	char *raw;
	char *tmp;

	rb_create(&sota_refs, sota_cmp, sizeof(struct sota_ref),
		  offsetof(struct sota_ref, node));

	raw = read_file_at(dirfd, fname);
	if (IS_ERR(raw))
		return PTR_ERR(raw);

	/* skip the first line */
	tmp = strchr(raw, '\n') + 1;

	ret = sig_load(tmp, sota_cols, ARRAY_LEN(sota_cols), __load_sota);

	free(raw);

	return ret;
}

int sig_lookup_pota(const char *ref, struct sig_pota_ref *data)
{
	struct str keyref = STR_STATIC_INITIALIZER(ref);
	struct pota_ref key = {
		.data.ref = &keyref,
	};
	struct pota_ref *found;

	found = rb_find(&pota_refs, &key, NULL);
	if (!found)
		return -ENOENT;

	data->ref = str_getref(found->data.ref);
	data->name = str_getref(found->data.name);
	data->locs = str_getref(found->data.locs);
	data->active = found->data.active;
	data->dxcc = found->data.dxcc;
	data->have_latlon = found->data.have_latlon;
	data->lat = found->data.lat;
	data->lon = found->data.lon;

	return 0;
}

int sig_lookup_sota(const char *ref, struct sig_sota_ref *data)
{
	struct str keyref = STR_STATIC_INITIALIZER(ref);
	struct sota_ref key = {
		.data.ref = &keyref,
	};
	struct sota_ref *found;

	found = rb_find(&sota_refs, &key, NULL);
	if (!found)
		return -ENOENT;

	data->ref = str_getref(found->data.ref);
	data->name = str_getref(found->data.name);
	data->points = found->data.points;

	return 0;
}
